package net.sf.itcb.common.portlet.vaadin;

import java.net.SocketException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import javax.portlet.PortletRequest;
import javax.portlet.PortletResponse;
import javax.portlet.RenderRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.itcb.common.business.core.ItcbApplicationContextHolder;
import net.sf.itcb.common.portlet.exceptions.PortletItcbException;
import net.sf.itcb.common.portlet.exceptions.PortletItcbExceptionMappingErrors;
import net.sf.itcb.common.portlet.vaadin.page.PageMappingProcessor;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;

import com.vaadin.terminal.ErrorMessage;
import com.vaadin.terminal.ParameterHandler;
import com.vaadin.terminal.SystemError;
import com.vaadin.terminal.Terminal;
import com.vaadin.terminal.URIHandler;
import com.vaadin.terminal.VariableOwner;
import com.vaadin.terminal.gwt.server.ChangeVariablesErrorEvent;
import com.vaadin.terminal.gwt.server.HttpServletRequestListener;
import com.vaadin.terminal.gwt.server.PortletRequestListener;
import com.vaadin.ui.AbstractComponent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Window;



/**
 * this abstract class is the base vaadin application portlet used by all the vaadin application
 * It manages common treatments :
 * <ul>
 * <li>user storage in a spring Context {@link org.springframework.security.core.context.SecurityContextHolder}</li>
 * <li>local storage in a spring Context {@link org.springframework.context.i18n.LocaleContextHolder}</li> 
 * <li>user storage in MDC context in order to be logged {@link org.slf4j.MDC}</li>
 * <li>create the application and all the views (VIEW, EDIT, HELP) when the application is run inside a portal or directly in an application server as a servlet</li>
 * <li>delegates the views contents to child classes</li>
 * </ul>
 * 
 * TODO (Feature request  3155816): Request in an portal free mode (running in Tomcat as servlet) is ok but 2 things has to be done :
 * <ul>
 * <li>Request are now RenderRequest but it has to be also possible to use HttpServletRequest</li>
 * <li>Users have to be mocked when we are in Servlet mode</li>
 * <li>Edit View and Help mode have to be displayed with a tabs screen instead of the 3 actual buttons.</li>
 * </ul>
 * In child class and all the other classes : Services, DAO... The user will be available by calling the spring context : SecurityContextHolder.getContext().getAuthentication().
 * @deprecated use ItcbVaadinApplication : will disappear at ITCB 0.6.0
 * @author Pierre Le Roux
 */
public abstract class AbstractItcbPortletApplication extends ItcbVaadinApplication implements HttpServletRequestListener, PortletRequestListener  {
	
	private static final long serialVersionUID = 1L;
	
	protected final Logger log = LoggerFactory.getLogger(getClass());
	
	protected Window mainWindow;
	
	protected String preferencesURL;
	
	protected Map<String,PageMappingProcessor> mapPageMappingProcessor;
		
	protected PageMappingProcessor currentPageMappingProcessor;
	
	protected String currentMode;
	
	// Application session. It is needed because portlet sessions are kept even if application is reloaded. It means that old portlet session is the same for new application in the same session.
	// It shouldn't.
	// With this session, vars are stored for the application life time. If application is reloaded, applicationSession is renewed
	private ConcurrentMap<String, Object> applicationSession;
	
	@Override
	public void onRequestStart(HttpServletRequest request,
			HttpServletResponse response) {	
		
		String itcbmode = request.getParameter("itcbmode");
		if(itcbmode == null) {
			itcbmode="view";
		}
		if(!isRunning() || (!itcbmode.equals(currentMode) )  ) {			
			initItcb(request, null, itcbmode);
		}
		
		
		// TODO What about the renderrequest test
	}
	
	@Override
	public void onRequestEnd(HttpServletRequest request,
			HttpServletResponse response) {
		// Nothing
		
	}
	
	

	@Override
	public void onRequestStart(PortletRequest request, PortletResponse response) {
		String itcbmode = request.getPortletMode().toString();
		if(!isRunning() || (itcbmode != null && !itcbmode.equals(currentMode) )  ) {			
			initItcb(null, request, itcbmode);
		}
		
		if(request instanceof RenderRequest) {
			currentPageMappingProcessor.refreshApplication();
		}
		
	}
	
	private void initItcb(HttpServletRequest servletRequest, PortletRequest portletRequest, String itcbmode) {
		applicationSession = new ConcurrentHashMap<String, Object>();
		mainWindow = new Window(getWindowName());
		setMainWindow(mainWindow);
		if(mapPageMappingProcessor==null) {
			mapPageMappingProcessor = new HashMap<String, PageMappingProcessor>();
		}
		
		PageMappingProcessor pmp;
		try {
			pmp = ItcbApplicationContextHolder.getContext().getBean(itcbmode + "ModePageMappingProcessor", PageMappingProcessor.class);
		}
		catch(NoSuchBeanDefinitionException nsbde) {
			throw new PortletItcbException(PortletItcbExceptionMappingErrors.COMMON_PORTLET_MISSING_PMP, "the "+ itcbmode +" associated mapping variable has to be set in spring context. The bean name has to be : " + itcbmode + "ModePageMappingProcessor");
		}
		
		mapPageMappingProcessor.put(itcbmode, pmp);	
		if(servletRequest != null) {
			pmp.setInitRequest(servletRequest);
		}
		else {
			pmp.setInitRequest(portletRequest);
		}
		currentPageMappingProcessor = pmp;
		
		
		currentMode=itcbmode;
		currentPageMappingProcessor.setApplication(AbstractItcbPortletApplication.this);
		
		try {
			if(currentPageMappingProcessor.isAutomaticDisplay()) {
				currentPageMappingProcessor.displayDefaultPage();
			}
		} 
		catch (Exception e) {
			try  {
				currentPageMappingProcessor.handleException(e);
			}
			catch(Exception errorInExceptionManagementException) {
				errorInExceptionManagementException.printStackTrace();
			}
		}
	}

	@Override
	public void onRequestEnd(PortletRequest request, PortletResponse response) {
		// Nothing
	}
	

	
	@Override
	public void init() {
		// Nothing
	}

	@Override
	public void terminalError(Terminal.ErrorEvent event) {
		final Throwable t = event.getThrowable();
        if (t instanceof SocketException) {
            // Most likely client browser closed socket
            return;
        }

        // Finds the original source of the error/exception
        Object owner = null;
        if (event instanceof VariableOwner.ErrorEvent) {
            owner = ((VariableOwner.ErrorEvent) event).getVariableOwner();
        } else if (event instanceof URIHandler.ErrorEvent) {
            owner = ((URIHandler.ErrorEvent) event).getURIHandler();
        } else if (event instanceof ParameterHandler.ErrorEvent) {
            owner = ((ParameterHandler.ErrorEvent) event).getParameterHandler();
        } else if (event instanceof ChangeVariablesErrorEvent) {
            owner = ((ChangeVariablesErrorEvent) event).getComponent();
        }

        // Shows the error in AbstractComponent
        if (owner instanceof AbstractComponent && !(owner instanceof Button) ) { // We don't want to display error on Button. Only on components
            if (t instanceof ErrorMessage) {
                ((AbstractComponent) owner).setComponentError((ErrorMessage) t);
            } else {
                ((AbstractComponent) owner)
                        .setComponentError(new SystemError(t));
            }
        }

        // also print the error on console
        // log.error("Terminal error:", t);
		
		try  {
			currentPageMappingProcessor.handleError(event);
		}
		catch(Exception errorInExceptionManagementException) {
			errorInExceptionManagementException.printStackTrace();
		}
	}

	
	public PageMappingProcessor getCurrentPageMappingProcessor() {
		return currentPageMappingProcessor;
	}
	

	protected String getWindowName() {
		return "";
	}
	
	public ConcurrentMap<String, Object> getApplicationSession() {
		return applicationSession;
	}
	
}
